; Wattmeter DC Leistungsmesser					
; 3x SA56 Multiplex								
; fr ATmega8									
; Bernhard.Erfurt@gmx.de						
; 06/2024										
;												

;												
; Display 320x240								
; 15 Zeilen Zeile 0-Zeile 14					
; Zeile0=Statuszeile (grau)						
; zur 30. Sekunde wird RTC ausgelesen			
;												
; Statuszeile=0 Farbe Grau						
;												

;												
; rot blinken + BEEP: TFT not found				
; rot+gelb    + BEEP: TWI-SCL auf GND			
; rot+grn    + BEEP: TWI-SDA auf GND			
;												

; grn: ADC										
; gelb: TWI										
; rot :	ERROR									

; alle 6 Minuten wird die Leistung				
; im EEPROM gespeichert							
; 0-6-12-18-24-30-36-42-48-54 (10 Werte/h)		
; 24h x 10 = 240 Bytes pro Tag					


; ATmega8 										
.include "m8def.inc"
.include "MACRO.asm"
; KONSTANTEN									
.equ FAKTOR_U=285
.equ FAKTOR_i=62
; TWI ADRESSEN									
.equ TWI_ADR_TFT					= 110	; 110 DISPLAY		
.equ TWI_ADR_RTC					= 104	; 104 RTC			
.equ TWI_ADR_EEPROM					= 84	; 84  EEPROM		
; Tasten										
.equ TASTE_MENUE_PLUS				=  1
.equ TASTE_MENUE_MINUS				=  2	
.equ TASTE_CLEAR					=  4	
.equ TASTE_RESERVE1					=  8	
.equ TASTE_RESERVE2					= 32
.equ TASTE_MINUS					= 64
.equ TASTE_PLUS						=128
; Schwellwerte									
.equ SCHWELLE_SPANNUNG				=160	; 16 V			
; GRAFIC-FARBEN									
.equ KONST_FARBE_TRANSPARENT		= 0
.equ KONST_FARBE_SCHWARZ			= 1
.equ KONST_FARBE_BLAUx				= 2  	;(Grundfarbe)	
.equ KONST_FARBE_ROT				= 3  	;(Grundfarbe) 	
.equ KONST_FARBE_GRUEN				= 4  	;(Grundfarbe)	
.equ KONST_FARBE_MAGENTA			= 5
.equ KONST_FARBE_GELB				= 7
.equ KONST_FARBE_WEISS				= 8		; !				
.equ KONST_FARBE_DUNKEL_GRAU		= 9
.equ KONST_FARBE_HELL_BLAU			= 15 
.equ KONST_FARBE_GRAU				= 16
; BEEP											
.equ BEEP_PIN_NR					= 6
.equ BEEP_DDR						= DDRB
.equ BEEP_PORT						= PORTB
; LED-ROT										
.equ ROT_PIN_NR						= 3
.equ ROT_DDR						= DDRB
.equ ROT_PORT						= PORTB
; LED-GELB										
.equ GELB_PIN_NR					= 4
.equ GELB_DDR						= DDRB
.equ GELB_PORT						= PORTB
; LED-GRUEN										
.equ GRUEN_PIN_NR					= 5
.equ GRUEN_DDR						= DDRB
.equ GRUEN_PORT						= PORTB
; LED-BLAU										
.equ BLAU_PIN_NR					= 0
.equ BLAU_DDR						= DDRB
.equ BLAU_PORT						= PORTB
; TWI-SDA										
.equ TWI_SDA_PIN_NR					= 4
.equ TWI_SDA_DDR					= DDRC
.equ TWI_SDA_PORT					= PORTC
; TWI-SCL										
.equ TWI_SCL_PIN_NR					= 5
.equ TWI_SCL_DDR					= DDRC
.equ TWI_SCL_PORT					= PORTC
; REGISTER										
.def ERROR			= R8
.def KOPIE_SREG 	= R9
.def KOPIE_TEMP 	= R10

.def COUNTER		= R11
.def TIMER0			= R12
.def NULL 			= R13
.def EINS 			= R14
.def VOLL			= R15
.def temp 			= R16
.def temp1 			= R17
.def temp2 			= R18
.def temp3 			= R19
.def temp4 			= R20
.def temp5 			= R21
.def temp6 			= R22
.def temp7 			= R23
.def temp8 			= R24
.def temp9 			= R25

; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						=SRAM_START		; 0x0060 START Adresse 		

.equ adr_ERROR						=SRAM_START+0

.equ adr_FREIGABE_ss				=SRAM_START+1
.equ adr_FREIGABE_mm				=SRAM_START+2
.equ adr_FREIGABE_hh				=SRAM_START+3
.equ adr_FREIGABE_TAG				=SRAM_START+4

.equ adr_VT1						=SRAM_START+5
.equ adr_VT2						=SRAM_START+6
.equ adr_VT3						=SRAM_START+7

.equ adr_ss							=SRAM_START+8
.equ adr_mm							=SRAM_START+9	
.equ adr_hh							=SRAM_START+10	
.equ adr_WOCHENTAG					=SRAM_START+11

.equ adr_TAG						=SRAM_START+12
.equ adr_MONAT						=SRAM_START+13
.equ adr_JAHR						=SRAM_START+14

.equ adr_MENUE						=SRAM_START+16
.equ adr_TASTE						=SRAM_START+17

.equ adr_Wmm_L						=SRAM_START+18	; Watt Minuten			
.equ adr_Wmm_H						=SRAM_START+19
.equ adr_Wh_L						=SRAM_START+20	; Watt Minuten			
.equ adr_Wh_H						=SRAM_START+21

.equ adr_REFRESH					=SRAM_START+22	; 1=neu zeichnen		


.equ adr_U_mom					 	=SRAM_START+23
.equ adr_U						 	=SRAM_START+24

.equ adr_i_mom						=SRAM_START+25
.equ adr_i							=SRAM_START+26

.equ adr_P_mom						=SRAM_START+30
.equ adr_P							=SRAM_START+31
.equ adr_P_MAX						=SRAM_START+32

.equ adr_ADC_U_L					=SRAM_START+40
.equ adr_ADC_U_H					=SRAM_START+41
.equ adr_ADC_i_L					=SRAM_START+42
.equ adr_ADC_i_H					=SRAM_START+43

.equ adr_COUNTER					=SRAM_START+44

.equ adr_DIAGRAMM_TAG				=SRAM_START+50
.equ adr_DIAGRAMM_XL				=SRAM_START+51
.equ adr_DIAGRAMM_XH				=SRAM_START+52

.equ adr_TFT_ZEIGER					=SRAM_START+60

.equ adr_DUMMY_L					=SRAM_START+65
.equ adr_DUMMY_H					=SRAM_START+66

.equ adr_ZEIGER_U					=SRAM_START+70
.equ adr_ZEIGER_i					=SRAM_START+71
.equ adr_ZEIGER_P					=SRAM_START+72

.equ adr_EEPROM_ZEIGER_L			=SRAM_START+80	; EEPROM-ADRESSE		
.equ adr_EEPROM_ZEIGER_H			=SRAM_START+81

.equ adr_OFFSET_TAG_L				=SRAM_START+90	; TAG-Offset fr SAVE	
.equ adr_OFFSET_TAG_H				=SRAM_START+91

.equ adr_OFFSET_DIAGRAMM_L			=SRAM_START+92	; TAG-Offset fr Diagramm
.equ adr_OFFSET_DIAGRAMM_H			=SRAM_START+93


.equ adr_GRAFIC_PUFFER_256			=SRAM_START+100


.equ adr_TWI_BEREICH				=SRAM_START+400


.equ adr_DATA_U64					=SRAM_START+600	; Datenfeld				
.equ adr_DATA_i64					=SRAM_START+700	; Datenfeld				
.equ adr_DATA_P64					=SRAM_START+800	; Datenfeld				

.equ adr_SRAM_CHECK					=SRAM_START+943	; SRAM CHECK muss immer "S" sein	

; EEPROM-BELEGUNG ATmega8						

; TWI EEPROM									
.equ adr_TWI_EEP_P_MAX				=100			; Leistung-max							

.equ adr_TWI_EEP_Wmm				=200			; Wmm, zuerst H							

.equ adr_TWI_EEP_DATA_U_MO			=1000			; 60 pro h x 24 = 1.440 Bytes pro Tag	
.equ adr_TWI_EEP_DATA_U_Di			=3000
.equ adr_TWI_EEP_DATA_U_Mi			=5000
.equ adr_TWI_EEP_DATA_U_Do			=7000
.equ adr_TWI_EEP_DATA_U_Fr			=9000
.equ adr_TWI_EEP_DATA_U_Sa			=11000
.equ adr_TWI_EEP_DATA_U_So			=13000

.equ adr_TWI_EEP_DATA_i_MO			=15000			; 60 pro h x 24 = 1.440 Bytes pro Tag	
.equ adr_TWI_EEP_DATA_i_Di			=17000
.equ adr_TWI_EEP_DATA_i_Mi			=19000
.equ adr_TWI_EEP_DATA_i_Do			=21000
.equ adr_TWI_EEP_DATA_i_Fr			=23000
.equ adr_TWI_EEP_DATA_i_Sa			=25000
.equ adr_TWI_EEP_DATA_i_So			=27000

.equ adr_TWI_EEP_DATA_P_MO			=29000			; 60 pro h x 24 = 1.440 Bytes pro Tag	
.equ adr_TWI_EEP_DATA_P_Di			=31000
.equ adr_TWI_EEP_DATA_P_Mi			=33000
.equ adr_TWI_EEP_DATA_P_Do			=35000
.equ adr_TWI_EEP_DATA_P_Fr			=37000
.equ adr_TWI_EEP_DATA_P_Sa			=39000
.equ adr_TWI_EEP_DATA_P_So			=41000			; Ende: 42.439							

.equ adr_TWI_EEP_DATA_JAHR			=50000			; Wh(Tageswerte) 12x31x2=744 Bytes zuerst H	


.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   ; Timer1 Overflow Handler           	
rjmp TIM0_OVF   		; Timer0 Overflow Handler           					
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	
;-------------------------------------------------------------------------------
; Unkorrekten Interrupt Aufruf 													
INTERRUPT_ERROR:
	ROT_ON
	GELB_OFF
	GRUEN_OFF
	BEEP_ON
	rcall wait_100ms
	ROT_OFF
	GELB_OFF
	GRUEN_OFF
	BEEP_OFF
	rcall wait_100ms
	rjmp INTERRUPT_ERROR
;-------------------------------------------------------------------------------
RESET:                        
	ldi temp, LOW(RAMEND)					; STACK initialisieren								
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
	rcall WDR_INITIALISIERUNG				; WDR_INITIALISIERUNG								
	rcall SRAM_CLEAR						; SRAM lschen										
	rcall STARTWERTE						; Startwerte										
	rcall PORT_INITIALISIERUNG				; PORT_CLEAR										
	rcall BEEP_KURZ
	rcall TWI_INI
	rcall TFT_INI
	rcall TFT_CLEAR
	rcall ANZEIGE_BEGRUESSUNG
	rcall WAIT_1s
	rcall TWI_EEPROM_READ_P_MAX				; P max load										
;	rcall TWI_RTC_SET						; Uhr stellen nach Standardeinstellungen			
	rcall TWI_RTC_READ
	rcall TWI_SCAN_CHECK
	rcall WAIT_100ms
	rcall ADC_INITIALISIERUNG				; ADC_INITIALISIERUNG								
	rcall TIMER0_INITIALISIERUNG_ms			; TIMER-Tasten										
	rcall TIMER_AKTIVIEREN					; TIMER												
	rcall PROGRAMM_CHECK					; CHECK												
	sei										; Interrupts freigeben				
;-------------------------------------------------------------------------------
	rcall TFT_CLEAR
	rcall TFT_CURSOR_OFF
;rcall TWI_EEPROM_TEST
	rcall BEEP_KURZ
	rcall WAIT_100ms
	rcall BEEP_KURZ
	ROT_OFF
	GELB_OFF
	GRUEN_OFF
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	rcall ADC_U
	rcall ADC_i
	rcall BERECHNUNG_P
	rcall BERECHNUNG_P_MAX
	rcall TWI_BERECHNUNG_EEPROM_ZEIGER_hh_mm; aus hh und mm	

	rcall TWI_BERECHNUNG_EEPROM_TAG_OFFSET	; aus Wochentag	
	rcall MENUE
	rcall PROGRAMM_CHECK					; CHECK			
	rcall TASTEN_WAIT
;-------------------------------------------------------------------------------
	LDS temp,(adr_FREIGABE_TAG)				; Freigabe ?	
	tst temp
	brne LOOP_TAG
	LDS temp,(adr_FREIGABE_hh)				; Freigabe ?	
	tst temp
	brne LOOP_hh
	LDS temp,(adr_FREIGABE_mm)				; Freigabe ?	
	tst temp
	brne LOOP_mm
	LDS temp,(adr_FREIGABE_ss)				; Freigabe ?	
	tst temp
	brne LOOP_ss
	rjmp LOOP
;-------------------------------------------------------------------------------
; TAG																			
LOOP_TAG:
	STS(adr_FREIGABE_TAG),NULL				; Null			
	STS(adr_REFRESH),EINS					; Refresh		
	rcall TFT_CLEAR							; Clear			
	rcall TWI_EEPROM_SAVE_DATENFELD_Wh		; Datenfeld		
	STS (adr_Wmm_L),NULL
	STS (adr_Wmm_H),NULL
	STS (adr_Wh_L) ,NULL
	STS (adr_Wh_H) ,NULL
	rcall TWI_RTC_WRITE_Wmm					; Wattminuten	
rjmp LOOP
;-------------------------------------------------------------------------------
; Stunde																		
LOOP_hh:
	STS(adr_FREIGABE_hh),NULL				; Null			
	STS(adr_REFRESH),EINS					; Refreh		
	rcall TFT_CLEAR							; Clear			
rjmp LOOP
;-------------------------------------------------------------------------------
; Minute																		
LOOP_mm:
	STS(adr_FREIGABE_mm),NULL				; Null			
;-------------------------------------------------------------------------------
; Wattminute in Wh																
	LDS temp,(adr_P)
	LDS temp1,(adr_Wmm_L)					; load			
	LDS temp2,(adr_Wmm_H)
	add temp1,temp							; P addieren	
	adc temp2,NULL
	STS (adr_Wmm_L),temp1					; Wmm save		
	STS (adr_Wmm_H),temp2
	ldi temp3,60
	rcall DIVISION_16_8						; durch 60		
	STS (adr_Wh_L),temp1					; Wh save		
	STS (adr_Wh_H),temp2
;-------------------------------------------------------------------------------
	rcall TWI_RTC_WRITE_Wmm					; Wattminuten	
	rcall WAIT_1s
	rcall TWI_EEPROM_SAVE_DATENFELD			; Datenfeld		
;-------------------------------------------------------------------------------
	LDS temp,(adr_hh)
	cpi temp,0								; Stunde=0 ?	
	brne LOOP_EXIT
	LDS temp,(adr_mm)
	cpi temp,1								; Minute=1 ?	
	brne LOOP_EXIT
	rcall TWI_EEPROM_TAG_CLEAR				; EEPROM-TAG-CLEAR	
;-------------------------------------------------------------------------------
; RESET durch WDR																
	rcall TFT_CLEAR
LOOP_mm_RESET:
	rjmp LOOP_mm_RESET
;-------------------------------------------------------------------------------
LOOP_EXIT:
rjmp LOOP
;-------------------------------------------------------------------------------
; SEKUNDE, bei 30 RTC															
LOOP_ss:
	STS(adr_FREIGABE_ss),NULL				; Null			
	; Umrechnung Wmm in Wh		
	LDS temp1,(adr_Wmm_L)					; load			
	LDS temp2,(adr_Wmm_H)
	ldi temp3,60
	rcall DIVISION_16_8						; durch 60		
	STS (adr_Wh_L),temp1					; Wh save		
	STS (adr_Wh_H),temp2
	; 30s RTC			
	LDS temp,(adr_ss)
	cpi temp,30								; 30. Sekunde RTC read
	brne LOOP_ss_EXIT
	rcall TWI_RTC_READ
LOOP_ss_EXIT:
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
BERECHNUNG_P:
	LDS temp1,(adr_U)
	LDS temp2,(adr_i)
	mul temp1,temp2
	mov temp1,R0
	mov temp2,R1
	ldi temp3,100
	rcall DIVISION_16_8 		; durch 100, P in temp1	
; P save																		
	STS(adr_P_mom),temp1		; save					
; Mittelwert-save																
	LDS temp,(adr_ZEIGER_P)		; Zeiger load	
	ldi ZL,low (adr_DATA_P64) 	; Z				
	ldi ZH,high(adr_DATA_P64)
	add ZL,temp					; addieren		
	adc ZH,NULL
	inc temp					; Zeiger +1		
	andi temp, 0b00111111
	STS(adr_ZEIGER_P),temp		; Zeiger save	
	ST Z+,temp1					; save			
; skip																			
	LDS temp,(adr_DATA_P64+63)
	tst temp
	brne BERECHNUNG_P_w
	LDS temp,(adr_P_mom)	; load			
	STS(adr_P),temp			; save			
	ret
; Mittewert																		
BERECHNUNG_P_w:
	ldi ZL,low (adr_DATA_P64); Z				
	ldi ZH,high(adr_DATA_P64)
	clr temp1
	clr temp2
	ldi temp9,64
BERECHNUNG_P_s:	
	LD temp,Z+
	add temp1,temp
	adc temp2,NULL
	dec temp9
	brne BERECHNUNG_P_s

	ldi temp3,low (23)		; aufrunden		
	ldi temp4,high(23)
	add temp1,temp3
	adc temp2,temp4
	LSR temp2				; :2			
	ROR temp1
	LSR temp2				; :4			
	ROR temp1
	LSR temp2				; :8			
	ROR temp1
	LSR temp2				; :16			
	ROR temp1
	LSR temp2				; :32			
	ROR temp1
	LSR temp2				; :64			
	ROR temp1
; p-save																		
	STS(adr_P),temp1		; save			

	ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
BERECHNUNG_P_MAX:
	LDS temp1,(adr_P_mom)
	LDS temp2,(adr_P_MAX)
	cp temp1,temp2
	brlo BERECHNUNG_P_MAX_KLEINER
	breq BERECHNUNG_P_MAX_GLEICH
	brsh BERECHNUNG_P_MAX_GROSSER
;-------------------------------------------------------------------------------
BERECHNUNG_P_MAX_KLEINER:
	ret
;-------------------------------------------------------------------------------
BERECHNUNG_P_MAX_GLEICH:
	STS(adr_P_MAX),temp1		; save					
	ret
;-------------------------------------------------------------------------------
BERECHNUNG_P_MAX_GROSSER:
	STS(adr_P_MAX),temp1
	rcall TWI_EEPROM_SAVE_P_MAX	; save					
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "ADC.asm"
.include "TIMER.asm"
.include "LED.asm"
.include "TWI.asm"
.include "TWI_TFT.asm"
.include "TWI_RTC_DS1307.asm"
.include "ANZEIGE.asm"
.include "TEXTE.asm"
.include "EEPROM.asm"
.include "TWI_EEPROM.asm"
.include "ZAHLEN.asm"
.include "ASCII.asm"
.include "MENUE.asm"
.include "DIAGRAMM.asm"
.include "DIAGRAMM_EEPROM.asm"
.include "DIAGRAMM_JAHR.asm"
.include "SONSTIGES.asm"
.include "WAIT.asm"
;-------------------------------------------------------------------------------
; Begrungstext						
TEXT_BEGRUESSUNG_1:
.db "Solar-TFT",0
TEXT_BEGRUESSUNG_2:
.db "TFT DISPLAY 240x320 EA eDIPTFT32-ATP ",0
TEXT_BEGRUESSUNG_3:
.db "ATmega8 / 1MHz ",0
TEXT_BEGRUESSUNG_4:
.db "320x240 40x15 Zeilen ",0
TEXT_BEGRUESSUNG_5:
.db "bernhard.erfurt@gmx.de ",0
TEXT_BEGRUESSUNG_6:
.db "22.07.2024 ",0

